/***************************************************************************
                          contactlist.c  -  description
                             -------------------
    begin                : Sun Apr 8 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 /*
 This file implements the contact list functionality.
 */

 #include "contactlist.h"

 /*
 cl_aux_compose_entry

 This function composes a contact list entry from the given dtails.
 It handles all memory allocations.
 */

 centry *cl_aux_compose_entry(char *cname, unsigned long cuin)
 {
 		centry *new_entry = (centry *) malloc(sizeof(centry));
 		
 		if (!new_entry) return NULL;
 		
 		if (!(new_entry->name = (char *) malloc(strlen(cname)+1)))
 		{
 			free(new_entry);
 			return NULL;
 		}
 		
 		strcpy(new_entry->name, cname);
 		new_entry->uin = cuin;
 		
 		return new_entry;
 }

 /*
 cl_aux_decompose_entry

 This function decomposes a contact list entry, freeing all allocated memory.
 */

 void cl_aux_decompose_entry(centry *ce)
 {
 		if (ce)
 		{
 			if (ce->name) free(ce->name);
 			free(ce);
 		}
 		
 		return;
 }

 /*
 cl_aux_compare_entry

 This function compares a contact list entry with a given name and uin.
 If the requested name is NULL it matches everything.
 If the requested uin is 0, it matches everything.
 */

 int cl_aux_compare_entry(centry *ce, char *cname, unsigned long cuin)
 {
 		if (cname)
 			if (strcmp(ce->name, cname))
 				return 0; // No match
 		
 		if (cuin)
 			if (ce->uin != cuin)
 				return 0; // No match
 				
 		return 1; // Match
 }


 /*
 cl_add

 This function adds a contact to the contact list.
 Note that the name string is copied (and therefore it should not
 be allocated beforehand).
 The function returns a pointer to the new contact.
 */

 centry *cl_add(clist *list, char *cname, unsigned long cuin)
 {
 		centry *new_entry = cl_aux_compose_entry(cname, cuin);
 		
 		if (!new_entry) return NULL;
 		
 		new_entry->next = list->first;
 		list->first = new_entry;
 		
 		return new_entry;
 }


 /*
 cl_find

 Finds an entry in the list.
 */

 centry *cl_find(clist *list, char *cname, unsigned long cuin)
 {
 		centry *current;
 		
 		for(current=list->first; current; current = current->next)
 			if (cl_aux_compare_entry(current, cname, cuin))
 				return current;
 				
 		return NULL;
 }



 /*
 cl_delete

 Searches for a certain contact and deletes its entry from the list.
 An entry can be indicated by name, uin or both.
 To specify a search by uin only, set the name field to NULL.
 To specify a search by name only, set the uin field to 0.
 */
 centry *cl_delete(clist *list, char *cname, unsigned long cuin)
 {
 		centry *previous, *current;
 		
 		if (!(list->first)) return NULL;
 		
 		if (cl_aux_compare_entry(list->first, cname, cuin))
 		{
 			current = list->first;
 			list->first = list->first->next;
 			cl_aux_decompose_entry(current);
 			return list->first;
 		}

 		/* Going over the whole list */
  		
 		for(previous=list->first, current=previous->next; current ; previous = current, current=current->next)
 			if (cl_aux_compare_entry(current, cname, cuin))
 			{
 				previous->next = current->next;
 				cl_aux_decompose_entry(current);
 				return previous->next;
 			}
 			
 		return NULL;
 }


 /*
 cl_read

 Reads a complete contact list from a file.
 The file is opened beforehand but it is NOT locked (the lock is
 implemented in this function).
 The file format is very simple, each entry is represented
 by two lines: the first contains the uin and the second contains
 the name.
 The name must not exceed MAX_CNAME_SIZE characters.
 */

 void cl_read(clist *list, FILE *in_file)
 {
 	char temp[MAX_CNAME_SIZE+2];
 	char rname[MAX_CNAME_SIZE+2];
 	unsigned long ruin;
 	
 	list->first = NULL;
 	
 	flock(fileno(in_file), LOCK_SH); // Locking the file for reading
 	
 	while (fgets(temp, MAX_CNAME_SIZE+2, in_file))
 	{
// 			temp[strlen(temp)-1] = '\0';
 			sscanf(temp, "%lu", &ruin);
 			fgets(rname, MAX_CNAME_SIZE+1, in_file);
 			rname[strlen(rname)-1] = '\0'; // Remove newline from name
 			cl_add(list, rname, ruin);
 	}
 	
 	flock(fileno(in_file), LOCK_UN);
 	
 	return;
 	
 }


 /*
 cl_write

 This function writes the contact list to a given file.
 This function handles the file locking.
 */

 void cl_write(clist *list, FILE *out_file)
 {
 	centry *current;
 	
 	flock(fileno(out_file), LOCK_EX);
 	
 	for(current=list->first; current; current = current->next)
 	{
 		fprintf(out_file, "%lu\n", current->uin);
 		fprintf(out_file, "%s\n", current->name);
 	}

 	flock(fileno(out_file), LOCK_UN);
	
 	return; 	

 }

 /*
 cl_init

 Initializes a contact list.
 */

 void cl_init(clist *list)
 {
 	list->first = NULL;
 	return;
 }

 /*
 cl_clean

 Cleans all entries in a contact list.
 */

 void cl_clean(clist *list)
 {
 	centry *current, *suc;
 	
 	current=list->first;
 	
 	while (current)
 	{
 		suc = current->next;
 		cl_aux_decompose_entry(current);
 		current = suc;
 	}
 	
 	list->first = NULL;
 	
 	return;

 }

